' IDE-Voreinstellungen:
' Option Strict On
' Option Explicit On

Imports System.ComponentModel

''' <summary>
''' erweitertes TabControl: Tabpage-Selektion wenn Maus berm Reiter, Farbanpassung des
''' Reiters an die Tabpage, Mglichkeit, Tabpages zu disablen, RiderClicked-Event
''' </summary>
Public Class TabControlEx : Inherits TabControl

   Public Class RiderClickedEventArgs : Inherits EventArgs
      Public ReadOnly TabPage As TabPage
      Public ReadOnly Index As Integer
      Public Sub New(ByVal Index As Integer, ByVal TabPage As TabPage)
         Me.Index = Index
         Me.TabPage = TabPage
      End Sub
   End Class
   'Da die TabPages schon in OnMouseMove selektiert werden, ist der Klick auf die Reiter frei,
   ' um als User-Eingabe ausgewertet zu werden - man gewinnt quasi einen Button auf dem Reiter
   Public Event RiderClicked As EventHandler(Of RiderClickedEventArgs)

   Private _Brush As New SolidBrush(Color.AliceBlue)
   Private ReadOnly _Disableds As New List(Of Control)

   Public Sub New()
      Me.DrawMode = TabDrawMode.OwnerDrawFixed
   End Sub

   Protected Overrides Sub OnClick(ByVal e As EventArgs)
      'MyBase.OnClick(e)                                 'Original-Click deaktivieren
      Dim Indx As Integer = GetTabRider(Me.PointToClient(Control.MousePosition))
      If Indx < 0 OrElse Not IsTabEnabled(Indx) Then Return
      RaiseEvent RiderClicked(Me, New RiderClickedEventArgs(Indx, TabPages(Indx)))
   End Sub

   Protected Overrides Sub OnMouseMove(ByVal e As MouseEventArgs)
      MyBase.OnMouseMove(e)
      Dim Indx As Integer = GetTabRider(e.Location)
      If Indx < 0 OrElse Me.SelectedIndex = Indx OrElse Not IsTabEnabled(Indx) Then Return
      Me.SelectedIndex = Indx
   End Sub

   Protected Overrides Sub OnSelecting(ByVal e As TabControlCancelEventArgs)
      If e.TabPage IsNot Nothing Then e.Cancel = _Disableds.Contains(e.TabPage)
      MyBase.OnSelecting(e)
   End Sub

   Protected Overrides Sub Dispose(ByVal disposing As Boolean)
      If disposing Then
         _Brush.Dispose()
      End If
      MyBase.Dispose(disposing)
   End Sub

   ''' <summary>zeichnet die Reiter im Stil der zugehrigen Tabpage</summary>
   Protected Overrides Sub OnDrawItem(ByVal e As System.Windows.Forms.DrawItemEventArgs)
      MyBase.OnDrawItem(e)
      Const tff As TextFormatFlags = _
         TextFormatFlags.HorizontalCenter Or TextFormatFlags.VerticalCenter
      Dim tp As TabPage = MyBase.TabPages(e.Index)
      Dim rct As Rectangle = e.Bounds
      Dim g As Graphics = e.Graphics
      If _Disableds.Contains(tp) Then
         _Brush.Color = SystemColors.Control
         g.FillRectangle(_Brush, rct)
         ControlPaint.DrawStringDisabled(g, tp.Text, Me.Font, Color.White, rct, tff)
      Else
         _Brush.Color = tp.BackColor
         g.FillRectangle(_Brush, rct)
         TextRenderer.DrawText(g, tp.Text, Me.Font, rct, tp.ForeColor, tff)
      End If
   End Sub

   Private Function GetTabRider(ByVal Pt As Point) As Integer
      For I As Integer = 0 To Me.TabPages.Count - 1
         If Me.GetTabRect(I).Contains(Pt) Then Return I
      Next
      Return -1
   End Function

   Public Property IsTabEnabled(ByVal Index As Integer) As Boolean
      Get
         Return Not _Disableds.Contains(MyBase.TabPages(Index))
      End Get
      Set(ByVal Value As Boolean)
         IsTabEnabled(MyBase.TabPages(Index)) = Value
      End Set
   End Property

   Public Property IsTabEnabled(ByVal Tp As TabPage) As Boolean
      Get
         Return Not _Disableds.Contains(Tp)
      End Get
      Set(ByVal Value As Boolean)
         If _Disableds.Contains(Tp) <> Value Then Return
         'Trick: "TabPage.Enabled" ist in der Intellisense gar nicht
         ' vorhanden. Trotzdem ist sie da (von Control geerbt), und verwendbar
         Tp.Enabled = Value
         If Value Then
            _Disableds.Remove(Tp)
         Else
            _Disableds.Add(Tp)
         End If
         Me.Invalidate(New Rectangle(0, 0, Me.Width, Me.GetTabRect(0).Height))
      End Set
   End Property
End Class